@@ -3,25 +3,32 @@ package spoke
 import (
 	"context"
 	"errors"
+	"fmt"
 	"time"
 
-	"github.com/open-cluster-management/addon-framework/pkg/lease"
+	"github.com/open-cluster-management/submariner-addon/pkg/cloud"
 	"github.com/spf13/cobra"
+	"open-cluster-management.io/addon-framework/pkg/lease"
+	workclient "open-cluster-management.io/api/client/work/clientset/versioned"
 
-	addonclient "github.com/open-cluster-management/api/client/addon/clientset/versioned"
-	addoninformers "github.com/open-cluster-management/api/client/addon/informers/externalversions"
-	configclient "github.com/stolostron/submariner-addon/pkg/client/submarinerconfig/clientset/versioned"
-	configinformers "github.com/stolostron/submariner-addon/pkg/client/submarinerconfig/informers/externalversions"
-	"github.com/stolostron/submariner-addon/pkg/helpers"
-	"github.com/stolostron/submariner-addon/pkg/spoke/submarineragent"
+	configclient "github.com/open-cluster-management/submariner-addon/pkg/client/submarinerconfig/clientset/versioned"
+	configinformers "github.com/open-cluster-management/submariner-addon/pkg/client/submarinerconfig/informers/externalversions"
+	"github.com/open-cluster-management/submariner-addon/pkg/helpers"
+	"github.com/open-cluster-management/submariner-addon/pkg/spoke/submarineragent"
+	addonclient "open-cluster-management.io/api/client/addon/clientset/versioned"
+	addoninformers "open-cluster-management.io/api/client/addon/informers/externalversions"
 
 	"github.com/openshift/library-go/pkg/controller/controllercmd"
 
+	"k8s.io/apimachinery/pkg/api/meta"
 	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/client-go/discovery"
 	"k8s.io/client-go/dynamic"
 	"k8s.io/client-go/dynamic/dynamicinformer"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes"
+	"k8s.io/client-go/rest"
+	"k8s.io/client-go/restmapper"
 	"k8s.io/client-go/tools/clientcmd"
 )
 
@@ -104,6 +111,21 @@ func (o *AgentOptions) RunAgent(ctx context.Context, controllerContext *controll
 		return err
 	}
 
+	hubClient, err := kubernetes.NewForConfig(hubRestConfig)
+	if err != nil {
+		return err
+	}
+
+	restMapper, err := buildRestMapper(controllerContext.KubeConfig)
+	if err != nil {
+		return err
+	}
+
+	workClient, err := workclient.NewForConfig(controllerContext.KubeConfig)
+	if err != nil {
+		return err
+	}
+
 	addOnInformers := addoninformers.NewSharedInformerFactoryWithOptions(addOnHubKubeClient, 10*time.Minute, addoninformers.WithNamespace(o.ClusterName))
 	configInformers := configinformers.NewSharedInformerFactoryWithOptions(configHubKubeClient, 10*time.Minute, configinformers.WithNamespace(o.ClusterName))
 
@@ -114,41 +136,26 @@ func (o *AgentOptions) RunAgent(ctx context.Context, controllerContext *controll
 	submarinerConfigController := submarineragent.NewSubmarinerConfigController(
 		o.ClusterName,
 		spokeKubeClient,
-		addOnHubKubeClient,
 		configHubKubeClient,
 		spokeKubeInformers.Core().V1().Nodes(),
 		addOnInformers.Addon().V1alpha1().ManagedClusterAddOns(),
 		configInformers.Submarineraddon().V1alpha1().SubmarinerConfigs(),
-		controllerContext.EventRecorder,
-	)
+		cloud.NewProviderFactory(restMapper, spokeKubeClient, workClient, spokeDynamicClient, hubClient),
+		controllerContext.EventRecorder)
 
 	gatewaysStatusController := submarineragent.NewGatewaysStatusController(
 		o.ClusterName,
 		addOnHubKubeClient,
-		addOnInformers.Addon().V1alpha1().ManagedClusterAddOns(),
 		spokeKubeInformers.Core().V1().Nodes(),
 		controllerContext.EventRecorder,
 	)
 
-	deploymentStatusController := submarineragent.NewDeploymentStatusController(
-		o.ClusterName,
-		o.InstallationNamespace,
-		addOnHubKubeClient,
-		addOnInformers.Addon().V1alpha1().ManagedClusterAddOns(),
-		spokeKubeInformers.Apps().V1().DaemonSets(),
-		spokeKubeInformers.Apps().V1().Deployments(),
-		dynamicInformers.ForResource(subscriptionGVR),
-		controllerContext.EventRecorder,
-	)
+	deploymentStatusController := submarineragent.NewDeploymentStatusController(o.ClusterName, o.InstallationNamespace,
+		addOnHubKubeClient, spokeKubeInformers.Apps().V1().DaemonSets(), spokeKubeInformers.Apps().V1().Deployments(),
+		dynamicInformers.ForResource(subscriptionGVR), controllerContext.EventRecorder)
 
-	connectionsStatusController := submarineragent.NewConnectionsStatusController(
-		o.ClusterName,
-		o.InstallationNamespace,
-		addOnHubKubeClient,
-		addOnInformers.Addon().V1alpha1().ManagedClusterAddOns(),
-		dynamicInformers.ForResource(submarinerGVR),
-		controllerContext.EventRecorder,
-	)
+	connectionsStatusController := submarineragent.NewConnectionsStatusController(o.ClusterName, addOnHubKubeClient,
+		dynamicInformers.ForResource(submarinerGVR), controllerContext.EventRecorder)
 
 	go addOnInformers.Start(ctx.Done())
 	go configInformers.Start(ctx.Done())
@@ -171,3 +178,17 @@ func (o *AgentOptions) RunAgent(ctx context.Context, controllerContext *controll
 	<-ctx.Done()
 	return nil
 }
+
+func buildRestMapper(restConfig *rest.Config) (meta.RESTMapper, error) {
+	discoveryClient, err := discovery.NewDiscoveryClientForConfig(restConfig)
+	if err != nil {
+		return nil, fmt.Errorf("error creating discovery client: %v", err)
+	}
+
+	groupResources, err := restmapper.GetAPIGroupResources(discoveryClient)
+	if err != nil {
+		return nil, fmt.Errorf("error retrieving API group resources: %v", err)
+	}
+
+	return restmapper.NewDiscoveryRESTMapper(groupResources), nil
+}
